@@ -3,32 +3,40 @@
  * Copyright (c) 2023, SWITCH */
 /* See LICENSE for licensing information. */
 
-#include <sys/time.h>
-#include <stdint.h>
-#include <pthread.h>
-#include <regex.h>
-#include <netinet/in.h>
+#ifndef _RADSECPROXY_H
+#define _RADSECPROXY_H
+
+#include "gconfig.h"
+#include "hostport.h"
 #include "list.h"
 #include "radmsg.h"
-#include "gconfig.h"
 #include "rewrite.h"
-#include "hostport.h"
+#include <netinet/in.h>
+#include <pthread.h>
+#include <regex.h>
+#include <stdint.h>
+#include <sys/time.h>
 
 #include <openssl/asn1.h>
 
 #define DEBUG_LEVEL 2
 
-#define CONFIG_MAIN SYSCONFDIR"/radsecproxy.conf"
+#define CONFIG_MAIN SYSCONFDIR "/radsecproxy.conf"
 
 /* MAX_REQUESTS must be 256 due to Radius' 8 bit ID field */
 #define MAX_REQUESTS 256
 #define MAX_LOSTRQS 16
 #define REQUEST_RETRY_INTERVAL 5
 #define REQUEST_RETRY_COUNT 2
-#define DUPLICATE_INTERVAL REQUEST_RETRY_INTERVAL * REQUEST_RETRY_COUNT
+#define DUPLICATE_INTERVAL REQUEST_RETRY_INTERVAL *REQUEST_RETRY_COUNT
 #define MAX_CERT_DEPTH 5
 #define STATUS_SERVER_PERIOD 25
 #define IDLE_TIMEOUT 300
+#define PSK_MIN_LENGTH 16
+#define RSP_SECRET_LEN_WARN 10
+/* Older OpenSSL API had a 256 byte limit; keep this limit to maximize compatibility*/
+#define PSK_ID_MAX_LENGTH 256
+#define RSP_TLS_REKEY_INTERVAL 3600
 
 /* Target value for stack size.
  * Some platforms might define higher minimums in PTHREAD_STACK_MIN. */
@@ -74,10 +82,10 @@ enum rsp_server_state {
 };
 
 enum rsp_statsrv {
-	RSP_STATSRV_OFF = 0,
-	RSP_STATSRV_ON,
-	RSP_STATSRV_MINIMAL,
-	RSP_STATSRV_AUTO
+    RSP_STATSRV_OFF = 0,
+    RSP_STATSRV_ON,
+    RSP_STATSRV_MINIMAL,
+    RSP_STATSRV_AUTO
 };
 
 struct options {
@@ -89,17 +97,18 @@ struct options {
     uint32_t ttlattrtype[2];
     uint8_t addttl;
     uint8_t loglevel;
-	uint8_t logtid;
-	uint8_t logfullusername;
+    uint8_t logtid;
+    uint8_t logfullusername;
     uint8_t loopprevention;
-	enum rsp_mac_type log_mac;
-	uint8_t *log_key;
+    enum rsp_mac_type log_mac;
+    uint8_t *log_key;
     enum rsp_fticks_reporting_type fticks_reporting;
     enum rsp_mac_type fticks_mac;
     uint8_t *fticks_key;
     uint8_t ipv4only;
     uint8_t ipv6only;
     uint8_t sni;
+    uint8_t verifyeap;
 };
 
 struct commonprotoopts {
@@ -110,7 +119,7 @@ struct commonprotoopts {
 struct request {
     struct timeval created;
     uint32_t refcount;
-	pthread_mutex_t refmutex;
+    pthread_mutex_t refmutex;
     uint8_t *buf, *replybuf;
     int buflen, replybuflen;
     struct radmsg *msg;
@@ -138,6 +147,8 @@ struct gqueue {
 };
 
 struct clsrvconf {
+    uint8_t shallow;
+    struct clsrvconf *parent;
     char *name;
     uint8_t type; /* RAD_UDP/RAD_TLS/RAD_TCP */
     const struct protodefs *pdef;
@@ -150,6 +161,9 @@ struct clsrvconf {
     uint8_t *secret;
     int secret_len;
     char *tls;
+    char *pskid;
+    uint8_t *pskkey;
+    int pskkeylen;
     struct list *matchcertattrs;
     char **confmatchcertattrs;
     char *confrewritein;
@@ -177,6 +191,8 @@ struct clsrvconf {
     uint8_t sni;
     char *sniservername;
     long dtlsmtu;
+    uint8_t reqmsgauth;
+    uint8_t reqmsgauthproxy;
 };
 
 #include "tlscommon.h"
@@ -185,11 +201,12 @@ struct client {
     struct clsrvconf *conf;
     int sock;
     SSL *ssl;
-	pthread_mutex_t lock;
+    pthread_mutex_t lock;
     struct request *rqs[MAX_REQUESTS];
     struct gqueue *replyq;
     struct sockaddr *addr;
     time_t expiry; /* for udp */
+    struct timeval tlsnewkey;
 };
 
 struct server {
@@ -201,6 +218,7 @@ struct server {
     uint8_t clientrdgone;
     struct timeval connecttime;
     struct timeval lastreply;
+    struct timeval tlsnewkey;
     enum rsp_server_state state;
     uint8_t lostrqs;
     char *dynamiclookuparg;
@@ -208,7 +226,7 @@ struct server {
     struct timeval lastrcv;
     struct rqout *requests;
     uint8_t newrq;
-	uint8_t conreset;
+    uint8_t conreset;
     pthread_mutex_t newrq_mutex;
     pthread_cond_t newrq_cond;
 };
@@ -240,9 +258,9 @@ struct protodefs {
     uint8_t duplicateintervaldefault;
     void (*setprotoopts)(struct commonprotoopts *);
     char **(*getlistenerargs)(void);
-    void *(*listener)(void*);
+    void *(*listener)(void *);
     int (*connecter)(struct server *, int, int);
-    void *(*clientconnreader)(void*);
+    void *(*clientconnreader)(void *);
     int (*clientradput)(struct server *, unsigned char *, int);
     void (*addclient)(struct client *);
     void (*addserverextra)(struct clsrvconf *);
@@ -252,6 +270,7 @@ struct protodefs {
 
 struct clsrvconf *find_clconf(uint8_t type, struct sockaddr *addr, struct list_node **cur, struct hostportres **hp);
 struct clsrvconf *find_srvconf(uint8_t type, struct sockaddr *addr, struct list_node **cur);
+struct list *find_all_clconf(uint8_t type, struct sockaddr *addr, struct list_node *cur, struct hostportres **hp);
 struct clsrvconf *find_clconf_type(uint8_t type, struct list_node **cur);
 struct client *addclient(struct clsrvconf *conf, uint8_t lock);
 void removelockedclient(struct client *client);
@@ -260,11 +279,15 @@ struct gqueue *newqueue(void);
 struct request *newrequest(void);
 void freerq(struct request *rq);
 int radsrv(struct request *rq);
-void replyh(struct server *server, uint8_t *buf, int buflen);
+int timeouth(struct server *server);
+int closeh(struct server *server);
+int replyh(struct server *server, uint8_t *buf, int buflen);
 struct addrinfo *resolve_hostport_addrinfo(uint8_t type, char *hostport);
 uint8_t *radattr2ascii(struct tlv *attr); /* TODO: mv this to radmsg? */
 extern pthread_attr_t pthread_attr;
 
+#endif /* _RADSECPROXY_H */
+
 /* Local Variables: */
 /* c-file-style: "stroustrup" */
 /* End: */
