@@ -3,27 +3,27 @@
  * Copyright (c) 2023, SWITCH */
 /* See LICENSE for licensing information. */
 
-#include <signal.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <string.h>
-#include <unistd.h>
 #include <limits.h>
-#ifdef SYS_SOLARIS9
+#include <netdb.h>
+#include <netinet/in.h>
+#include <signal.h>
+#include <string.h>
+#include <sys/socket.h>
+#include <unistd.h>
+#ifdef SYS_SOLARIS
 #include <fcntl.h>
 #endif
-#include <sys/time.h>
-#include <sys/types.h>
-#include <poll.h>
-#include <ctype.h>
-#include <sys/wait.h>
-#include <arpa/inet.h>
-#include <regex.h>
-#include <pthread.h>
-#include "radsecproxy.h"
 #include "hostport.h"
 #include "list.h"
+#include "radsecproxy.h"
+#include <arpa/inet.h>
+#include <ctype.h>
+#include <poll.h>
+#include <pthread.h>
+#include <regex.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 
 #ifdef RADPROT_TCP
 #include "debug.h"
@@ -38,24 +38,24 @@ void tcpsetsrcres(void);
 
 static const struct protodefs protodefs = {
     "tcp",
-    NULL, /* secretdefault */
-    SOCK_STREAM, /* socktype */
-    "1812", /* portdefault */
-    0, /* retrycountdefault */
-    0, /* retrycountmax */
-    REQUEST_RETRY_INTERVAL * REQUEST_RETRY_COUNT, /* retryintervaldefault */
-    60, /* retryintervalmax */
-    DUPLICATE_INTERVAL, /* duplicateintervaldefault */
-    setprotoopts, /* setprotoopts */
-    getlistenerargs, /* getlistenerargs */
-    tcplistener, /* listener */
-    tcpconnect, /* connecter */
-    tcpclientrd, /* clientconnreader */
-    clientradputtcp, /* clientradput */
-    NULL, /* addclient */
-    NULL, /* addserverextra */
-    tcpsetsrcres, /* setsrcres */
-    NULL /* initextra */
+    NULL,                                        /* secretdefault */
+    SOCK_STREAM,                                 /* socktype */
+    "1812",                                      /* portdefault */
+    0,                                           /* retrycountdefault */
+    0,                                           /* retrycountmax */
+    REQUEST_RETRY_INTERVAL *REQUEST_RETRY_COUNT, /* retryintervaldefault */
+    60,                                          /* retryintervalmax */
+    DUPLICATE_INTERVAL,                          /* duplicateintervaldefault */
+    setprotoopts,                                /* setprotoopts */
+    getlistenerargs,                             /* getlistenerargs */
+    tcplistener,                                 /* listener */
+    tcpconnect,                                  /* connecter */
+    tcpclientrd,                                 /* clientconnreader */
+    clientradputtcp,                             /* clientradput */
+    NULL,                                        /* addclient */
+    NULL,                                        /* addserverextra */
+    tcpsetsrcres,                                /* setsrcres */
+    NULL                                         /* initextra */
 };
 
 static struct addrinfo *srcres = NULL;
@@ -76,7 +76,7 @@ static char **getlistenerargs(void) {
 
 void tcpsetsrcres(void) {
     if (!srcres)
-	srcres =
+        srcres =
             resolvepassiveaddrinfo(protoopts ? protoopts->sourcearg : NULL,
                                    AF_UNSPEC, NULL, protodefs.socktype);
 }
@@ -91,13 +91,13 @@ int tcpconnect(struct server *server, int timeout, int reconnect) {
 
     debug(DBG_DBG, "tcpconnect: %s to %s", reconnect ? "reconnecting" : "initial connection", server->conf->name);
     pthread_mutex_lock(&server->lock);
-
     if (server->state == RSP_SERVER_STATE_CONNECTED)
         server->state = RSP_SERVER_STATE_RECONNECTING;
+    pthread_mutex_unlock(&server->lock);
 
-    if(server->conf->source) {
+    if (server->conf->source) {
         source = resolvepassiveaddrinfo(server->conf->source, AF_UNSPEC, NULL, protodefs.socktype);
-        if(!source)
+        if (!source)
             debug(DBG_WARN, "tcpconnect: could not resolve source address to bind for server %s, using default", server->conf->name);
     }
 
@@ -108,20 +108,19 @@ int tcpconnect(struct server *server, int timeout, int reconnect) {
             close(server->sock);
         server->sock = -1;
 
-        pthread_mutex_unlock(&server->lock);
         wait = connect_wait(start, server->connecttime, firsttry);
         gettimeofday(&now, NULL);
         if (timeout && (now.tv_sec - start.tv_sec) + wait > timeout) {
             debug(DBG_DBG, "tcpconnect: timeout");
-            if (source) freeaddrinfo(source);
+            if (source)
+                freeaddrinfo(source);
             return 0;
         }
-        if (wait) debug(DBG_INFO, "Next connection attempt to %s in %lds", server->conf->name, wait);
+        if (wait)
+            debug(DBG_INFO, "Next connection attempt to %s in %lds", server->conf->name, wait);
         sleep(wait);
         firsttry = 0;
 
-        pthread_mutex_lock(&server->lock);
-
         for (entry = list_first(server->conf->hostports); entry; entry = list_next(entry)) {
             hp = (struct hostportres *)entry->data;
             debug(DBG_INFO, "tcpconnect: trying to open TCP connection to server %s (%s port %s)", server->conf->name, hp->host, hp->port);
@@ -139,8 +138,9 @@ int tcpconnect(struct server *server, int timeout, int reconnect) {
             enable_keepalive(server->sock);
         break;
     }
-    server->state = RSP_SERVER_STATE_CONNECTED;
     gettimeofday(&server->connecttime, NULL);
+    pthread_mutex_lock(&server->lock);
+    server->state = RSP_SERVER_STATE_CONNECTED;
     server->lostrqs = 0;
     pthread_mutex_unlock(&server->lock);
     pthread_mutex_lock(&server->newrq_mutex);
@@ -148,7 +148,8 @@ int tcpconnect(struct server *server, int timeout, int reconnect) {
     pthread_cond_signal(&server->newrq_cond);
     pthread_mutex_unlock(&server->newrq_mutex);
 
-    if (source) freeaddrinfo(source);
+    if (source)
+        freeaddrinfo(source);
     return 1;
 }
 
@@ -159,21 +160,21 @@ int tcpreadtimeout(int s, unsigned char *buf, int num, int timeout) {
     struct pollfd fds[1];
 
     if (s < 0)
-	return -1;
+        return -1;
     /* make socket non-blocking? */
     for (len = 0; len < num; len += cnt) {
         fds[0].fd = s;
         fds[0].events = POLLIN;
-	ndesc = poll(fds, 1, timeout? timeout * 1000 : -1);
-	if (ndesc < 1)
-	    return ndesc;
+        ndesc = poll(fds, 1, timeout ? timeout * 1000 : -1);
+        if (ndesc < 1)
+            return ndesc;
 
-    if (fds[0].revents & (POLLERR | POLLHUP | POLLNVAL) ) {
-        return -1;
-    }
-	cnt = read(s, buf + len, num - len);
-	if (cnt <= 0)
-	    return -1;
+        if (fds[0].revents & (POLLERR | POLLHUP | POLLNVAL)) {
+            return -1;
+        }
+        cnt = read(s, buf + len, num - len);
+        if (cnt <= 0)
+            return -1;
     }
     return num;
 }
@@ -207,6 +208,7 @@ int radtcpget(int s, int timeout, uint8_t **buf) {
     if (cnt < 1) {
         debug(DBG_DBG, cnt ? "radtcpget: connection lost" : "radtcpget: timeout");
         free(*buf);
+        *buf = NULL;
         return cnt;
     }
     debug(DBG_DBG, "radtcpget: got %d bytes", len);
@@ -217,51 +219,48 @@ int clientradputtcp(struct server *server, unsigned char *rad, int radlen) {
     int cnt;
     struct clsrvconf *conf = server->conf;
 
-    if (server->state != RSP_SERVER_STATE_CONNECTED)
-        return 0;
     if (radlen <= 0) {
         debug(DBG_ERR, "clientradputtcp: invalid buffer (length)");
         return 0;
     }
+    pthread_mutex_lock(&server->lock);
+    if (server->state != RSP_SERVER_STATE_CONNECTED) {
+        pthread_mutex_unlock(&server->lock);
+        return 0;
+    }
     if ((cnt = write(server->sock, rad, radlen)) <= 0) {
         debug(DBG_ERR, "clientradputtcp: write error");
+        pthread_mutex_unlock(&server->lock);
         return 0;
     }
     debug(DBG_DBG, "clientradputtcp: Sent %d bytes, Radius packet of length %zu to TCP peer %s", cnt, radlen, conf->name);
+    pthread_mutex_unlock(&server->lock);
     return 1;
 }
 
 void *tcpclientrd(void *arg) {
     struct server *server = (struct server *)arg;
     unsigned char *buf;
-    struct timeval now;
     int len = 0;
 
     for (;;) {
-        len = radtcpget(server->sock, server->conf->retryinterval * (server->conf->retrycount+1), &buf);
-        if (!buf || len <= 0) {
-            if (len < 0 || (server->lostrqs && server->conf->statusserver!=RSP_STATSRV_OFF) ) {
-                if (len < 0)
-                    debug (DBG_WARN, "tlsclientrd: connection to server %s lost", server->conf->name);
-                else if (server->lostrqs)
-                    debug (DBG_WARN, "tlsclientrd: server %s did not respond, closing connection.", server->conf->name);
-                if (server->dynamiclookuparg)
+        len = radtcpget(server->sock, server->conf->retryinterval * (server->conf->retrycount + 1), &buf);
+        if (buf && len > 0) {
+            if (!replyh(server, buf, len))
+                if (closeh(server))
                     break;
-                tcpconnect(server, 0, 1);
-            }
-            if (server->dynamiclookuparg) {
-                gettimeofday(&now, NULL);
-                if (now.tv_sec - server->lastreply.tv_sec > IDLE_TIMEOUT) {
-                    debug(DBG_INFO, "tcpclientrd: idle timeout for %s", server->conf->name);
-                    break;
-                }
-            }
-            continue;
+            buf = NULL;
+        } else if (len == 0) {
+            if (timeouth(server))
+                break;
+        } else {
+            if (closeh(server))
+                break;
         }
-
-        replyh(server, buf, len);
-        buf = NULL;
     }
+    shutdown(server->sock, SHUT_RDWR);
+    close(server->sock);
+
     server->clientrdgone = 1;
     pthread_mutex_lock(&server->newrq_mutex);
     pthread_cond_signal(&server->newrq_cond);
@@ -279,29 +278,29 @@ void *tcpserverwr(void *arg) {
     debug(DBG_DBG, "tcpserverwr: starting for %s", addr2string(client->addr, tmp, sizeof(tmp)));
     replyq = client->replyq;
     for (;;) {
-	pthread_mutex_lock(&replyq->mutex);
-	while (!list_first(replyq->entries)) {
-	    if (client->sock >= 0) {
-		debug(DBG_DBG, "tcpserverwr: waiting for signal");
-		pthread_cond_wait(&replyq->cond, &replyq->mutex);
-		debug(DBG_DBG, "tcpserverwr: got signal");
-	    }
-	    if (client->sock < 0) {
-		/* s might have changed while waiting */
-		pthread_mutex_unlock(&replyq->mutex);
-		debug(DBG_DBG, "tcpserverwr: exiting as requested");
-		pthread_exit(NULL);
-	    }
-	}
-	reply = (struct request *)list_shift(replyq->entries);
-	pthread_mutex_unlock(&replyq->mutex);
-	cnt = write(client->sock, reply->replybuf, reply->replybuflen);
-	if (cnt > 0)
-	    debug(DBG_DBG, "tcpserverwr: sent %d bytes, Radius packet of length %d to %s",
-		  cnt, reply->replybuflen, addr2string(client->addr, tmp, sizeof(tmp)));
-	else
-	    debug(DBG_ERR, "tcpserverwr: write error for %s", addr2string(client->addr, tmp, sizeof(tmp)));
-	freerq(reply);
+        pthread_mutex_lock(&replyq->mutex);
+        while (!list_first(replyq->entries)) {
+            if (client->sock >= 0) {
+                debug(DBG_DBG, "tcpserverwr: waiting for signal");
+                pthread_cond_wait(&replyq->cond, &replyq->mutex);
+                debug(DBG_DBG, "tcpserverwr: got signal");
+            }
+            if (client->sock < 0) {
+                /* s might have changed while waiting */
+                pthread_mutex_unlock(&replyq->mutex);
+                debug(DBG_DBG, "tcpserverwr: exiting as requested");
+                pthread_exit(NULL);
+            }
+        }
+        reply = (struct request *)list_shift(replyq->entries);
+        pthread_mutex_unlock(&replyq->mutex);
+        cnt = write(client->sock, reply->replybuf, reply->replybuflen);
+        if (cnt > 0)
+            debug(DBG_DBG, "tcpserverwr: sent %d bytes, Radius packet of length %d to %s",
+                  cnt, reply->replybuflen, addr2string(client->addr, tmp, sizeof(tmp)));
+        else
+            debug(DBG_ERR, "tcpserverwr: write error for %s", addr2string(client->addr, tmp, sizeof(tmp)));
+        freerq(reply);
     }
 }
 
@@ -362,8 +361,8 @@ void *tcpservernew(void *arg) {
     s = *(int *)arg;
     free(arg);
     if (getpeername(s, (struct sockaddr *)&from, &fromlen)) {
-	debug(DBG_DBG, "tcpservernew: getpeername failed, exiting");
-	goto exit;
+        debug(DBG_DBG, "tcpservernew: getpeername failed, exiting");
+        goto exit;
     }
     debug(DBG_WARN, "tcpservernew: incoming TCP connection from %s", addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)));
 
@@ -371,7 +370,7 @@ void *tcpservernew(void *arg) {
     if (conf) {
         client = addclient(conf, 1);
         if (client) {
-            if(conf->keepalive)
+            if (conf->keepalive)
                 enable_keepalive(s);
             client->sock = s;
             client->addr = addr_copy((struct sockaddr *)&from);
@@ -388,34 +387,29 @@ exit:
     pthread_exit(NULL);
 }
 
-void *tcplistener(void *arg) {
+void tcpaccept(int s) {
+    int *s_arg;
     pthread_t tcpserverth;
-    int s, *sp = (int *)arg, *s_arg = NULL;
-    struct sockaddr_storage from;
-    socklen_t fromlen = sizeof(from);
 
-    listen(*sp, 128);
-
-    for (;;) {
-	s = accept(*sp, (struct sockaddr *)&from, &fromlen);
-	if (s < 0) {
-	    debug(DBG_WARN, "accept failed");
-	    continue;
-	}
-        s_arg = malloc(sizeof(s));
-        if (!s_arg)
-            debugx(1, DBG_ERR, "malloc failed");
-        *s_arg = s;
-	if (pthread_create(&tcpserverth, &pthread_attr, tcpservernew, (void *) s_arg)) {
-	    debug(DBG_ERR, "tcplistener: pthread_create failed");
-            free(s_arg);
-	    shutdown(s, SHUT_RDWR);
-	    close(s);
-	    continue;
-	}
-	pthread_detach(tcpserverth);
+    s_arg = malloc(sizeof(s));
+    if (!s_arg) {
+        debug(DBG_ERR, "tcpaccept: malloc failed");
+        return;
     }
-    free(sp);
+    *s_arg = s;
+    if (pthread_create(&tcpserverth, &pthread_attr, tcpservernew, (void *)s_arg)) {
+        debug(DBG_ERR, "tcpaccept: pthread_create failed");
+        free(s_arg);
+        shutdown(s, SHUT_RDWR);
+        close(s);
+        return;
+    }
+    pthread_detach(tcpserverth);
+}
+
+void *tcplistener(void *arg) {
+    accepttcp(*(int *)arg, tcpaccept);
+    free(arg);
     return NULL;
 }
 #else
